@@ -15,7 +15,9 @@
 package main
 
 import (
-	"github.com/go-ping/ping"
+	"net"
+
+	"github.com/prometheus-community/pro-bing"
 
 	"github.com/go-kit/log/level"
 	"github.com/prometheus/client_golang/prometheus"
@@ -27,9 +29,9 @@ const (
 )
 
 var (
-	labelNames = []string{"ip", "host"}
+	labelNames = []string{"ip", "host", "source"}
 
-	pingResponseTtl = promauto.NewGaugeVec(
+	pingResponseTTL = promauto.NewGaugeVec(
 		prometheus.GaugeOpts{
 			Namespace: namespace,
 			Name:      "response_ttl",
@@ -45,15 +47,31 @@ var (
 		},
 		labelNames,
 	)
+	pingRecvErrors = promauto.NewCounter(
+		prometheus.CounterOpts{
+			Namespace: namespace,
+			Name:      "receive_errors_total",
+			Help:      "The number of errors when Pinger attempts to receive packets.",
+		},
+	)
+	pingSendErrors = promauto.NewCounterVec(
+		prometheus.CounterOpts{
+			Namespace: namespace,
+			Name:      "send_errors_total",
+			Help:      "The number of errors when Pinger attempts to send packets.",
+		},
+		labelNames,
+	)
 )
 
-func newPingResponseHistogram(buckets []float64) *prometheus.HistogramVec {
+func newPingResponseHistogram(buckets []float64, factor float64) *prometheus.HistogramVec {
 	return prometheus.NewHistogramVec(
 		prometheus.HistogramOpts{
-			Namespace: namespace,
-			Name:      "response_duration_seconds",
-			Help:      "A histogram of latencies for ping responses.",
-			Buckets:   buckets,
+			Namespace:                   namespace,
+			Name:                        "response_duration_seconds",
+			Help:                        "A histogram of latencies for ping responses.",
+			Buckets:                     buckets,
+			NativeHistogramBucketFactor: factor,
 		},
 		labelNames,
 	)
@@ -61,37 +79,53 @@ func newPingResponseHistogram(buckets []float64) *prometheus.HistogramVec {
 
 // SmokepingCollector collects metrics from the pinger.
 type SmokepingCollector struct {
-	pingers *[]*ping.Pinger
+	pingers *[]*probing.Pinger
 
 	requestsSent *prometheus.Desc
 }
 
-func NewSmokepingCollector(pingers *[]*ping.Pinger, pingResponseSeconds prometheus.HistogramVec) *SmokepingCollector {
+func NewSmokepingCollector(pingers *[]*probing.Pinger, pingResponseSeconds prometheus.HistogramVec) *SmokepingCollector {
 	for _, pinger := range *pingers {
 		// Init all metrics to 0s.
 		ipAddr := pinger.IPAddr().String()
-		pingResponseDuplicates.WithLabelValues(ipAddr, pinger.Addr())
-		pingResponseSeconds.WithLabelValues(ipAddr, pinger.Addr())
-		pingResponseTtl.WithLabelValues(ipAddr, pinger.Addr())
+		pingResponseDuplicates.WithLabelValues(ipAddr, pinger.Addr(), pinger.Source)
+		pingResponseSeconds.WithLabelValues(ipAddr, pinger.Addr(), pinger.Source)
+		pingResponseTTL.WithLabelValues(ipAddr, pinger.Addr(), pinger.Source)
+		pingSendErrors.WithLabelValues(ipAddr, pinger.Addr(), pinger.Source)
 
 		// Setup handler functions.
-		pinger.OnRecv = func(pkt *ping.Packet) {
-			pingResponseSeconds.WithLabelValues(pkt.IPAddr.String(), pkt.Addr).Observe(pkt.Rtt.Seconds())
-			pingResponseTtl.WithLabelValues(pkt.IPAddr.String(), pkt.Addr).Set(float64(pkt.Ttl))
+		pinger.OnRecv = func(pkt *probing.Packet) {
+			pingResponseSeconds.WithLabelValues(pkt.IPAddr.String(), pkt.Addr, pinger.Source).Observe(pkt.Rtt.Seconds())
+			pingResponseTTL.WithLabelValues(pkt.IPAddr.String(), pkt.Addr, pinger.Source).Set(float64(pkt.TTL))
 			level.Debug(logger).Log("msg", "Echo reply", "ip_addr", pkt.IPAddr,
-				"bytes_received", pkt.Nbytes, "icmp_seq", pkt.Seq, "time", pkt.Rtt, "ttl", pkt.Ttl)
+				"bytes_received", pkt.Nbytes, "icmp_seq", pkt.Seq, "time", pkt.Rtt, "ttl", pkt.TTL)
 		}
-		pinger.OnDuplicateRecv = func(pkt *ping.Packet) {
-			pingResponseDuplicates.WithLabelValues(pkt.IPAddr.String(), pkt.Addr).Inc()
+		pinger.OnDuplicateRecv = func(pkt *probing.Packet) {
+			pingResponseDuplicates.WithLabelValues(pkt.IPAddr.String(), pkt.Addr, pinger.Source).Inc()
 			level.Debug(logger).Log("msg", "Echo reply (DUP!)", "ip_addr", pkt.IPAddr,
-				"bytes_received", pkt.Nbytes, "icmp_seq", pkt.Seq, "time", pkt.Rtt, "ttl", pkt.Ttl)
+				"bytes_received", pkt.Nbytes, "icmp_seq", pkt.Seq, "time", pkt.Rtt, "ttl", pkt.TTL)
 		}
-		pinger.OnFinish = func(stats *ping.Statistics) {
+		pinger.OnFinish = func(stats *probing.Statistics) {
 			level.Debug(logger).Log("msg", "Ping statistics", "addr", stats.Addr,
 				"packets_sent", stats.PacketsSent, "packets_received", stats.PacketsRecv,
 				"packet_loss_percent", stats.PacketLoss, "min_rtt", stats.MinRtt, "avg_rtt",
 				stats.AvgRtt, "max_rtt", stats.MaxRtt, "stddev_rtt", stats.StdDevRtt)
 		}
+		pinger.OnRecvError = func(err error) {
+			if neterr, ok := err.(*net.OpError); ok {
+				if neterr.Timeout() {
+					// Ignore read timeout errors, these are handled by the pinger.
+					return
+				}
+			}
+			pingRecvErrors.Inc()
+			level.Debug(logger).Log("msg", "Error receiving packet", "error", err)
+		}
+		pinger.OnSendError = func(pkt *probing.Packet, err error) {
+			pingSendErrors.WithLabelValues(pkt.IPAddr.String(), pkt.Addr, pinger.Source).Inc()
+			level.Debug(logger).Log("msg", "Error sending packet", "ip_addr", pkt.IPAddr,
+				"bytes_received", pkt.Nbytes, "icmp_seq", pkt.Seq, "time", pkt.Rtt, "ttl", pkt.TTL, "error", err)
+		}
 	}
 
 	return &SmokepingCollector{
@@ -119,6 +153,7 @@ func (s *SmokepingCollector) Collect(ch chan<- prometheus.Metric) {
 			float64(stats.PacketsSent),
 			stats.IPAddr.String(),
 			stats.Addr,
+			pinger.Source,
 		)
 	}
 }
