@@ -94,6 +94,7 @@ type TargetGroup struct {
 	Network  string        `yaml:"network,omitempty"`
 	Protocol string        `yaml:"protocol,omitempty"`
 	Size     int           `yaml:"size,omitempty"`
+	Source   string        `yaml:"source,omitempty"`
 	// TODO: Needs work to fix MetricFamily consistency.
 	// Labels   map[string]string `yaml:"labels,omitempty"`
 }
@@ -101,18 +102,12 @@ type TargetGroup struct {
 // UnmarshalYAML implements the yaml.Unmarshaler interface.
 func (s *Config) UnmarshalYAML(unmarshal func(interface{}) error) error {
 	type plain Config
-	if err := unmarshal((*plain)(s)); err != nil {
-		return err
-	}
-	return nil
+	return unmarshal((*plain)(s))
 }
 
 // UnmarshalYAML implements the yaml.Unmarshaler interface.
 func (s *TargetGroup) UnmarshalYAML(unmarshal func(interface{}) error) error {
 	*s = DefaultTargetGroup
 	type plain TargetGroup
-	if err := unmarshal((*plain)(s)); err != nil {
-		return err
-	}
-	return nil
+	return unmarshal((*plain)(s))
 }
